<?php
/**
* phpCan v.1
*
* $Id$
*
* 2009 Copyright - phpCan is released under the GNU Affero GPL version 3
*
* More information at license.txt
*/

defined('ANS') or die();

class Image_library {
	protected $image;
	protected $info;

	private $image_types = array('gif', 'jpeg', 'png', 'swf', 'psd', 'bmp', 'tiff_ii', 'tiff_mm', 'jpc', 'jp2', 'jpx', 'jb2', 'swc', 'iff', 'wbmp', 'xbm', 'ico');


	/**
	* public function load (string $image)
	*
	* Load an image
	*/
	public function load ($image) {
		if ($data = @getImageSize($image)) {
			$extension = image_type_to_extension($data[2], false);
			$function = 'imagecreatefrom'.$extension;

			if (function_exists($function)) {
				$this->image = $function($image);

				$this->info = array(
					'file' => $image,
					'width' => $data[0],
					'height' => $data[1],
					'type' => $data[2],
					'mime' => $data['mime'],
					'format' => $extension,
				);
			}
		}
		return $this;
	}


	/**
	* public function unload (void)
	*
	* Destroy an image
	*/
	public function unload () {
		imagedestroy($this->image);
	}


	/**
	* public function save (string $filename)
	*
	* Save the image to file
	*/
	public function save ($filename = '') {
		$function = 'image'.$this->info['format'];

		if (function_exists($function)) {
			$filename = $filename ? $filename : $this->info['file'];

			$function($this->image, $filename);
		}

		return $this;
	}


	/**
	* public function resize (int $width, [int $height])
	*
	* Resize an image
	*/
	public function resize ($width, $height = 0) {
		if (!$width && !$height) {
			return false;
		}

		if ($width != 0 && ($height == 0 || ($this->info['width']/$width) > ($this->info['height']/$height))) {
			$new_width = $width;
			$new_height = floor(($width/$this->info['width']) * $this->info['height']);
		} else {
			$new_width = floor(($height/$this->info['height']) * $this->info['width']);
			$new_height = $height;
		}

		$tmp_image = imagecreatetruecolor($new_width, $new_height);

		imagecopyresampled($tmp_image, $this->image, 0, 0, 0, 0, $new_width, $new_height, $this->info['width'], $this->info['height']);

		$this->image = $tmp_image;

		$this->info['width'] = $new_width;
		$this->info['height'] = $new_height;

		return $this;
	}


	/**
	* public function crop (int $width, int $height, int $x, int $y)
	*
	* Crop an image
	*/
	public function crop ($width, $height, $x = 0, $y = 0) {
		$x = $this->position($x, $width, $this->info['width']);
		$y = $this->position($y, $height, $this->info['height']);

		$tmp_image = imagecreatetruecolor($width, $height);

		imagecopyresampled($tmp_image, $this->image, 0, 0, $x, $y, $this->info['width'], $this->info['height'], $this->info['width'], $this->info['height']);

		$this->image = $tmp_image;

		$this->info['width'] = $width;
		$this->info['height'] = $height;

		return $this;
	}


	/**
	* public function flip (void)
	*
	* Invert an image vertically
	*/
	public function flip () {
		$tmp_image = imagecreatetruecolor($this->info['width'], $this->info['height']);

		imagecopyresampled($tmp_image, $this->image, 0, 0, 0, ($this->info['height'] - 1), $this->info['width'], $this->info['height'], $this->info['width'], -$this->info['height']);

		$this->image = $tmp_image;

		return $this;
	}


	/**
	* public function flop (void)
	*
	* Invert an image horizontally
	*/
	public function flop () {
		$tmp_image = imagecreatetruecolor($this->info['width'], $this->info['height']);

		imagecopyresampled($tmp_image, $this->image, 0, 0, ($this->info['width'] - 1), 0, $this->info['width'], $this->info['height'], -$this->info['width'], $this->info['height']);

		$this->image = $tmp_image;

		return $this;
	}


	/**
	* public function zoomCrop (int $width, int $height)
	*
	* Crop an resize an image to specific dimmensions
	*/
	public function zoomCrop ($width, $height) {
		$width_resize = ($width/$this->info['width']) * 100;
		$height_resize = ($height/$this->info['height']) * 100;

		if ($width_resize < $height_resize) {
			$this->resize(0, $height);
		} else {
			$this->resize($width, 0);
		}

		$this->crop($width, $height, 'center', 'middle');

		return $this;
	}


	/**
	* public function show (void)
	*
	* Return the image
	*/
	public function show ($header = true) {

		//Show header mime-type
		if ($header && $this->info['mime']) {
			header('Content-Type: '.$this->info['mime']);
		}

		$function = 'image'.$this->info['format'];

		if (function_exists($function)) {
			$function($this->image);
		}

		die();
	}


	/**
	* public function rotate (int $degrees, [int $background])
	*
	* Rotate an image
	*/
	public function rotate ($degrees, $background = null) {
		$this->image = imagerotate($this->image, $degrees, $background);

		$this->info['width'] = imagesx($this->image);
		$this->info['height'] = imagesy($this->image);

		return $this;
	}


	/**
	* public function merge (string/object $image, [int $x], [int $y])
	*
	* Merge two images in one
	*/
	public function merge ($image, $x = 0, $y = 0) {
		if (!is_resource($image)) {
			if ($data = @getImageSize($image)) {
				$extension = image_type_to_extension($data[2], false);
				$function = 'imagecreatefrom'.$extension;

				if (function_exists($function)) {
					$image = $function($image);
				}
			}
		}

		$width = imagesx($image);
		$height = imagesy($image);

		if ($this->info['width'] > $width) {
			$width = $this->info['width'];
		}
		if ($this->info['height'] > $height) {
			$height = $this->info['height'];
		}

		$x = $this->position($x, $width, $this->info['width']);
		$y = $this->position($y, $height, $this->info['height']);

		$tmp_image = imagecreatetruecolor($width, $height);

		imagecopymerge($tmp_image, $this->image, 0, 0, 0, 0, $this->info['width'], $this->info['height'], 100);
		imagecopy($tmp_image, $image, 0, 0, 0, 0, $width, $height);

		$this->image = $tmp_image;

		$this->info['width'] = $width;
		$this->info['height'] = $height;

		return $this;
	}


	/**
	* public function convert (string $format)
	*
	* Convert an image to another format
	*/
	public function convert ($format) {
		$type = array_search($format, $this->image_types);

		if ($type === false) {
			return false;
		}

		$type++;

		$this->info['type'] = $type;
		$this->info['mime'] = image_type_to_mime_type($type);
		$this->info['format'] = image_type_to_extension($type);

		return $this;
	}
}
?>